@@ -26,6 +26,7 @@
 
 #import <NGObjWeb/NSException+HTTP.h>
 #import <NGObjWeb/WOContext+SoObjects.h>
+#import <NGExtensions/NGBase64Coding.h>
 #import <NGExtensions/NSNull+misc.h>
 #import <NGExtensions/NSObject+Logs.h>
 #import <NGExtensions/NSString+misc.h>
@@ -329,7 +330,7 @@ static NSString *inboxFolderName = @"INBOX";
   NSURL *nsURL;
   NSString *baseURLString, *urlString;
 
-  baseURLString = [[self imap4URL] absoluteString];
+  baseURLString = [self imap4URLString];
   urlString = [NSString stringWithFormat: @"%@%@/", baseURLString, [namespace stringByEscapingURL]];
   nsURL = [NSURL URLWithString: urlString];
   folders = [[self imap4Connection] allFoldersForURL: nsURL
@@ -555,10 +556,10 @@ static NSString *inboxFolderName = @"INBOX";
 					  nil];
 
           if (sieveFolderUTF8Encoding)
-            sievePath = [folderPath stringByDecodingImap4FolderName];
+            sievePath = [currentPath stringByDecodingImap4FolderName];
           else
-            sievePath = folderPath;
-          [folder setObject: [sievePath substringFromIndex: 1] forKey: @"sievePath"];
+            sievePath = currentPath;
+          [folder setObject: sievePath forKey: @"sievePath"];
 
           // Either add this new folder to its parent or the list of root folders
           [folders addObject: folder];
@@ -797,7 +798,7 @@ static NSString *inboxFolderName = @"INBOX";
       else
         guid = [[[result objectForKey: @"status"] objectForKey: currentFolder] objectForKey: @"x-guid"];
       
-      if (!guid || [guid isNull])
+      if (!guid || ![guid isNotNull])
         {
           // Don't generate a GUID for "Other users" and "Shared" namespace folders - user foldername instead
           if ((otherUsersFolderName && [currentFolder isEqualToString: otherUsersFolderName]) ||
@@ -1184,4 +1185,80 @@ static NSString *inboxFolderName = @"INBOX";
   return [[self _mailAccount] objectForKey: @"name"];
 }
 
+- (NSData *) certificate
+{
+  NSData *mailCertificate;
+  SOGoUserDefaults *ud;
+
+  mailCertificate = nil;
+  ud = [[context activeUser] userDefaults];
+
+  if ([nameInContainer isEqualToString: @"0"])
+    {
+      mailCertificate = [[ud stringForKey: @"SOGoMailCertificate"] dataByDecodingBase64];
+    }
+  else
+    {
+      int accountIdx;
+      NSArray* accounts;
+      NSDictionary *account, *security;
+
+      accountIdx = [nameInContainer intValue] - 1;
+      accounts = [ud auxiliaryMailAccounts];
+      if ([accounts count] > accountIdx)
+        {
+          account = [accounts objectAtIndex: accountIdx];
+          security = [account objectForKey: @"security"];
+          if (security && [[security objectForKey: @"certificate"] length])
+            {
+              mailCertificate = [[security objectForKey: @"certificate"] dataByDecodingBase64];
+            }
+        }
+    }
+
+  return mailCertificate;
+}
+
+- (void) setCertificate: (NSData *) theData
+{
+  SOGoUserDefaults *ud;
+
+  ud = [[context activeUser] userDefaults];
+
+  if ([nameInContainer isEqualToString: @"0"])
+    {
+      if ([theData length])
+        [ud setObject: [theData stringByEncodingBase64]  forKey: @"SOGoMailCertificate"];
+      else
+        [ud removeObjectForKey: @"SOGoMailCertificate"];
+    }
+  else
+    {
+      int accountIdx;
+      NSArray* accounts;
+      NSMutableDictionary *account, *security;
+
+      accountIdx = [nameInContainer intValue] - 1;
+      accounts = [ud auxiliaryMailAccounts];
+      if ([accounts count] > accountIdx)
+        {
+          account = [accounts objectAtIndex: accountIdx];
+          security = [account objectForKey: @"security"];
+          if (!security)
+            {
+              security = [NSMutableDictionary dictionary];
+              [account setObject: security forKey: @"security"];
+            }
+          if ([theData length])
+            [security setObject: [theData stringByEncodingBase64] forKey: @"certificate"];
+          else
+            [security removeObjectForKey: @"certificate"];
+          [ud setAuxiliaryMailAccounts: accounts];
+        }
+    }
+
+  [ud synchronize];
+}
+
+
 @end /* SOGoMailAccount */
